@@ -6,7 +6,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  *************************************************************************
  * The code here is based on code from Cryptlib 3.0 beta by Peter Gutmann.
@@ -61,9 +59,6 @@
  */
 
 #include <config.h>
-
-#ifdef USE_RNDW32
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -482,7 +477,18 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
      * worst-case estimate which is usually nowhere near the actual amount
      * required.  For example it may report that 128K of memory is required,
      * but only return 64K of data */
-    {	pPerfData =  m_alloc (cbPerfData);
+    if (getenv("GNUPG_RNDW32_NOPERF"))
+      {
+        static int shown;
+        
+        if (!shown)
+          {
+            shown = 1;
+            g10_log_info ("note: get performance data has been disabled\n");
+          }
+      }
+    else
+    {	pPerfData =  xmalloc (cbPerfData);
 	for (;;) {
 	    dwSize = cbPerfData;
 	    if ( debug_me )
@@ -499,14 +505,14 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
 	    }
 	    else if (status == ERROR_MORE_DATA) {
 		cbPerfData += PERFORMANCE_BUFFER_STEP;
-		pPerfData = m_realloc (pPerfData, cbPerfData);
+		pPerfData = xrealloc (pPerfData, cbPerfData);
 	    }
 	    else {
 		g10_log_debug ( "rndw32: get performance data problem\n");
 		break;
 	    }
 	}
-	m_free (pPerfData);
+	xfree (pPerfData);
     }
     /* Although this isn't documented in the Win32 API docs, it's necessary
        to explicitly close the HKEY_PERFORMANCE_DATA key after use (it's
@@ -700,5 +706,3 @@ rndw32_gather_random_fast( void (*add)(const void*, size_t, int), int requester
 
     return 0;
 }
-
-#endif /*USE_RNDW32*/
