@@ -196,6 +196,7 @@ maybe_create_keyring (char *filename, int force)
  * Flag 1 == force
  * Flag 2 == mark resource as primary
  * Flag 4 == This is a default resources
+ * Flag 8 == Readonly
  */
 int
 keydb_add_resource (const char *url, int flags, int secret)
@@ -204,10 +205,14 @@ keydb_add_resource (const char *url, int flags, int secret)
     const char *resname = url;
     char *filename = NULL;
     int force=(flags&1);
+    int read_only=!!(flags&8);
     int rc = 0;
     KeydbResourceType rt = KEYDB_RESOURCE_TYPE_NONE;
     void *token;
 
+    if (read_only)
+      force = 0;
+
     /* Do we have an URL?
      *	gnupg-ring:filename  := this is a plain keyring
      *	filename := See what is is, but create as plain keyring.
@@ -235,7 +240,7 @@ keydb_add_resource (const char *url, int flags, int secret)
     else
 	filename = xstrdup (resname);
 
-    if (!force)
+    if (!force && !read_only)
 	force = secret? !any_secret : !any_public;
 
     /* see whether we can determine the filetype */
