@@ -1,5 +1,5 @@
 /* keyring.c - keyring file handling
- * Copyright (C) 2001, 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2004, 2007, 2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -419,38 +419,43 @@ keyring_get_keyblock (KEYRING_HANDLE hd, KBNODE *ret_kb)
             if ( lastnode 
                  && lastnode->pkt->pkttype == PKT_SIGNATURE
                  && (pkt->pkt.ring_trust->sigcache & 1) ) {
-                /* this is a ring trust packet with a checked signature 
+                /* This is a ring trust packet with a checked signature 
                  * status cache following directly a signature paket.
-                 * Set the cache status into that signature packet */
+                 * Set the cache status into that signature packet.  */
                 PKT_signature *sig = lastnode->pkt->pkt.signature;
                 
                 sig->flags.checked = 1;
                 sig->flags.valid = !!(pkt->pkt.ring_trust->sigcache & 2);
             }
-            /* reset lastnode, so that we set the cache status only from
-             * the ring trust packet immediately folling a signature */
+            /* Reset LASTNODE, so that we set the cache status only
+             * from the ring trust packets immediately following
+             * signature packets.  */
             lastnode = NULL;
-        }
-        else {
-            node = lastnode = new_kbnode (pkt);
-            if (!keyblock)
-                keyblock = node;
-            else
-                add_kbnode (keyblock, node);
-
-            if ( pkt->pkttype == PKT_PUBLIC_KEY
-                 || pkt->pkttype == PKT_PUBLIC_SUBKEY
-                 || pkt->pkttype == PKT_SECRET_KEY
-                 || pkt->pkttype == PKT_SECRET_SUBKEY) {
-                if (++pk_no == hd->found.pk_no)
-                    node->flag |= 1;
-            }
-            else if ( pkt->pkttype == PKT_USER_ID) {
-                if (++uid_no == hd->found.uid_no)
-                    node->flag |= 2;
-            }
+	    free_packet(pkt);
+	    init_packet(pkt);
+            continue;
         }
 
+        node = lastnode = new_kbnode (pkt);
+        if (!keyblock)
+          keyblock = node;
+        else
+          add_kbnode (keyblock, node);
+        
+        if ( pkt->pkttype == PKT_PUBLIC_KEY
+             || pkt->pkttype == PKT_PUBLIC_SUBKEY
+             || pkt->pkttype == PKT_SECRET_KEY
+             || pkt->pkttype == PKT_SECRET_SUBKEY) 
+          {
+            if (++pk_no == hd->found.pk_no)
+              node->flag |= 1;
+          }
+        else if ( pkt->pkttype == PKT_USER_ID) 
+          {
+            if (++uid_no == hd->found.uid_no)
+              node->flag |= 2;
+          }
+        
         pkt = xmalloc (sizeof *pkt);
         init_packet(pkt);
     }
@@ -1177,11 +1182,11 @@ create_tmp_file (const char *template,
 	strcpy (stpcpy(tmpfname, template), EXTSEP_S "tmp");
     }
 # else /* Posix file names */
-    bakfname = xmalloc (strlen( template ) + 2);
-    strcpy (stpcpy (bakfname,template),"~");
+    bakfname = xmalloc (strlen (template) + sizeof (GNUPG_BAK_SFX) + 1);
+    strcpy (stpcpy (bakfname, template), GNUPG_BAK_SFX);
 
-    tmpfname = xmalloc (strlen( template ) + 5);
-    strcpy (stpcpy(tmpfname,template), EXTSEP_S "tmp");
+    tmpfname = xmalloc (strlen (template) + sizeof (GNUPG_TMP_SFX) + 1 );
+    strcpy (stpcpy(tmpfname, template), GNUPG_TMP_SFX);
 # endif /* Posix filename */
 
     /* Create the temp file with limited access */
@@ -1212,10 +1217,17 @@ static int
 rename_tmp_file (const char *bakfname, const char *tmpfname,
                  const char *fname, int secret )
 {
-  int rc=0;
+  int rc=G10ERR_GENERAL;
+
+  /* It's a secret keyring, so let's force a fsync just to be safe on
+     filesystems that may not sync data and metadata together
+     (ext4). */
+  if(secret && iobuf_ioctl(NULL,4,0,(char*)tmpfname)!=0)
+    goto fail;
 
   /* invalidate close caches*/
-  iobuf_ioctl (NULL, 2, 0, (char*)tmpfname );
+  if(iobuf_ioctl (NULL, 2, 0, (char*)tmpfname )!=0)
+    goto fail;
   iobuf_ioctl (NULL, 2, 0, (char*)bakfname );
   iobuf_ioctl (NULL, 2, 0, (char*)fname );
 
@@ -1245,15 +1257,7 @@ rename_tmp_file (const char *bakfname, const char *tmpfname,
                  tmpfname, fname, strerror(errno) );
       register_secured_file (fname);
       rc = G10ERR_RENAME_FILE;
-      if (secret)
-        {
-          log_info(_("WARNING: 2 files with confidential"
-                     " information exists.\n"));
-          log_info(_("%s is the unchanged one\n"), fname );
-          log_info(_("%s is the new one\n"), tmpfname );
-          log_info(_("Please fix this possible security flaw\n"));
-	}
-      return rc;
+      goto fail;
     }
 
   /* Now make sure the file has the same permissions as the original */
@@ -1275,6 +1279,17 @@ rename_tmp_file (const char *bakfname, const char *tmpfname,
 #endif
 
   return 0;
+
+ fail:
+  if(secret)
+    {
+      log_info(_("WARNING: 2 files with confidential information exists.\n"));
+      log_info(_("%s is the unchanged one\n"), fname );
+      log_info(_("%s is the new one\n"), tmpfname );
+      log_info(_("Please fix this possible security flaw\n"));
+    }
+
+  return rc;
 }
 
 
@@ -1384,7 +1399,20 @@ keyring_rebuild_cache (void *token,int noisy)
           log_error ("keyring_get_keyblock failed: %s\n", g10_errstr(rc));
           goto leave;
         }
-      assert (keyblock->pkt->pkttype == PKT_PUBLIC_KEY);
+      if ( keyblock->pkt->pkttype != PKT_PUBLIC_KEY)
+        {
+          /* We had a few reports about corrupted keyrings; if we have
+             been called directly from the command line we delete such
+             a keyblock instead of bailing out.  */
+          log_error ("unexpected keyblock found (pkttype=%d)%s\n",
+                     keyblock->pkt->pkttype, noisy? " - deleted":"");
+          if (noisy)
+            continue;
+          log_info ("Hint: backup your keys and try running `%s'\n",
+                    "gpg --rebuild-keydb-caches");
+          rc = G10ERR_INV_KEYRING;
+          goto leave;
+        }
 
       /* check all signature to set the signature's cache flags */
       for (node=keyblock; node; node=node->next)
