@@ -1,12 +1,12 @@
 /* http.c  -  HTTP protocol handler
- * Copyright (C) 1999, 2001, 2002, 2003, 2004,
- *               2005 Free Software Foundation, Inc.
+ * Copyright (C) 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
+ *               2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -70,7 +68,8 @@ static int remove_escapes( byte *string );
 static int insert_escapes( byte *buffer, const byte *string,
 					 const byte *special );
 static URI_TUPLE parse_tuple( byte *string );
-static int send_request( HTTP_HD hd, const char *auth, const char *proxy );
+static int send_request( HTTP_HD hd, const char *auth, const char *proxy,
+			 const char *srvtag, STRLIST headers);
 static byte *build_rel_path( PARSED_URI uri );
 static int parse_response( HTTP_HD hd );
 
@@ -150,7 +149,8 @@ make_radix64_string( const byte *data, size_t len )
 
 int
 http_open( HTTP_HD hd, HTTP_REQ_TYPE reqtype, const char *url,
-	   char *auth, unsigned int flags, const char *proxy )
+	   char *auth, unsigned int flags, const char *proxy,
+	   const char *srvtag, STRLIST headers )
 {
     int rc;
 
@@ -166,7 +166,7 @@ http_open( HTTP_HD hd, HTTP_REQ_TYPE reqtype, const char *url,
 
     rc = parse_uri( &hd->uri, url );
     if( !rc ) {
-	rc = send_request( hd, auth, proxy );
+        rc = send_request( hd, auth, proxy, srvtag, headers );
 	if( !rc ) {
 	    hd->fp_write = iobuf_sockopen( hd->sock , "w" );
 	    if( hd->fp_write )
@@ -234,11 +234,13 @@ http_wait_response( HTTP_HD hd, unsigned int *ret_status )
 
 int
 http_open_document( HTTP_HD hd, const char *document, char *auth,
-		    unsigned int flags, const char *proxy )
+		    unsigned int flags, const char *proxy, const char *srvtag,
+		    STRLIST headers )
 {
     int rc;
 
-    rc = http_open(hd, HTTP_REQ_GET, document, auth, flags, proxy );
+    rc = http_open(hd, HTTP_REQ_GET, document, auth, flags, proxy, srvtag,
+		   headers );
     if( rc )
 	return rc;
 
@@ -345,16 +347,27 @@ do_parse_uri( PARSED_URI uri, int only_local_part )
 	      }
 
 	    strlwr( p );
-	    uri->host = p;
-	    if( (p3=strchr( p, ':' )) ) {
-		*p3++ = 0;
-		uri->port = atoi( p3 );
-	    }
 
-	    uri->host = p;
+	    /* Handle a host of [IP] so that [IP:V6]:port works */
+	    if( *p == '[' && (p3=strchr( p, ']' )) )
+	      {
+		*p3++ = '\0';
+		/* worst case, uri->host should have length 0, points to \0 */
+		uri->host = p + 1;
+		p = p3;
+	      }
+	    else
+	      uri->host = p;
+
+	    if( (p3=strchr( p, ':' )) )
+	      {
+		*p3++ = '\0';
+		uri->port = atoi( p3 );
+	      }
+
 	    if( (n = remove_escapes( uri->host )) < 0 )
 		return G10ERR_BAD_URI;
-	    if( n != strlen( p ) )
+	    if( n != strlen( uri->host ) )
 		return G10ERR_BAD_URI; /* hostname with a Nul in it */
 	    p = p2 ? p2 : NULL;
 	}
@@ -509,7 +522,8 @@ parse_tuple( byte *string )
  * Returns 0 if the request was successful
  */
 static int
-send_request( HTTP_HD hd, const char *auth, const char *proxy )
+send_request( HTTP_HD hd, const char *auth, const char *proxy,
+	      const char *srvtag, STRLIST headers )
 {
     const byte *server;
     byte *request, *p;
@@ -546,7 +560,7 @@ send_request( HTTP_HD hd, const char *auth, const char *proxy )
 	release_parsed_uri( uri );
       }
     else
-      hd->sock = connect_server( server, port, hd->flags, hd->uri->scheme );
+      hd->sock = connect_server( server, port, hd->flags, srvtag );
 
     if(auth || hd->uri->auth)
       {
@@ -601,6 +615,19 @@ send_request( HTTP_HD hd, const char *auth, const char *proxy )
     xfree(p);
 
     rc = write_server( hd->sock, request, strlen(request) );
+
+    if(rc==0)
+      for(;headers;headers=headers->next)
+	{
+	  rc = write_server( hd->sock, headers->d, strlen(headers->d) );
+	  if(rc)
+	    break;
+
+	  rc = write_server( hd->sock, "\r\n", 2 );
+	  if(rc)
+	    break;
+	}
+
     xfree( request );
     xfree(proxy_authstr);
     xfree(authstr);
@@ -715,7 +742,7 @@ parse_response( HTTP_HD hd )
 
 #ifdef TEST
 static int
-start_server()
+start_server(void)
 {
     struct sockaddr_in mya;
     struct sockaddr_in peer;
@@ -827,7 +854,7 @@ connect_server( const char *server, ushort port, unsigned int flags,
 
 #ifdef USE_DNS_SRV
   /* Do the SRV thing */
-  if(flags&HTTP_FLAG_TRY_SRV && srvtag)
+  if(srvtag)
     {
       /* We're using SRV, so append the tags */
       if(1+strlen(srvtag)+6+strlen(server)+1<=MAXDNAME)
@@ -847,7 +874,9 @@ connect_server( const char *server, ushort port, unsigned int flags,
     {
       /* Either we're not using SRV, or the SRV lookup failed.  Make
 	 up a fake SRV record. */
-      srvlist=xmalloc_clear(sizeof(struct srventry));
+      srvlist=calloc(1,sizeof(struct srventry));
+      if(!srvlist)
+	return -1;
       srvlist->port=port;
       strncpy(srvlist->target,server,MAXDNAME);
       srvlist->target[MAXDNAME-1]='\0';
@@ -950,7 +979,7 @@ connect_server( const char *server, ushort port, unsigned int flags,
     }
 #endif /* !HAVE_GETADDRINFO */
 
-  xfree(srvlist);
+  free(srvlist);
 
   if(!connected)
     {
@@ -1064,7 +1093,7 @@ main(int argc, char **argv)
     }
     release_parsed_uri( uri ); uri = NULL;
 
-    rc = http_open_document( &hd, *argv, 0, NULL );
+    rc = http_open_document( &hd, *argv, NULL, 0, NULL, NULL, NULL );
     if( rc ) {
 	log_error("can't get `%s': %s\n", *argv, g10_errstr(rc));
 	return 1;
